Package: simplermarkdown
Title: Simple Engine for Generating Reports using R
Version: 0.0.6
Authors@R: 
    person(given = "Jan",
           family = "van der Laan",
           role = c("aut", "cre"),
           email = "r@eoos.dds.nl",
           comment = c(ORCID = "0000-0002-0693-1514"))
Description: Runs R-code present in a pandoc markdown file and 
  includes the resulting output in the resulting markdown file. This
  file can then be converted into any of the output formats 
  supported by pandoc. The package can also be used as an engine
  for writing package vignettes. 
BugReports: https://github.com/djvanderlaan/simplermarkdown/issues
URL: https://github.com/djvanderlaan/simplermarkdown
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.2.3
Imports: rjson, tools
Suggests: MASS
VignetteBuilder: simplermarkdown
SystemRequirements: Pandoc (http://pandoc.org) needs to installed and
        available in the search path.
NeedsCompilation: no
Packaged: 2023-03-24 07:55:56 UTC; eoos
Author: Jan van der Laan [aut, cre] (<https://orcid.org/0000-0002-0693-1514>)
Maintainer: Jan van der Laan <r@eoos.dds.nl>
Repository: CRAN
Date/Publication: 2023-03-24 15:00:02 UTC
Built: R 4.6.0; ; 2026-04-27 03:22:05 UTC; unix

# Generated by roxygen2: do not edit by hand

export(file_subs_ext)
export(format_copypaste)
export(format_traditional)
export(get_extensions)
export(markdown_block)
export(md_figure)
export(md_table)
export(mdtangle)
export(mdweave)
export(mdweave_to_html)
export(mdweave_to_pdf)
export(mdweave_to_tex)
export(output_eval)
export(output_figure)
export(output_raw)
export(output_shell)
export(output_str)
export(output_table)
export(raw_block)
export(run_and_capture)
importFrom(rjson,fromJSON)

# table
aggregate(iris[1:4], iris["Species"], mean)

# figure
pal <- hcl.colors(3, "Dark2")
plot(iris$Sepal.Width, iris$Sepal.Length, pch = 20, 
  col = pal[iris$Species], xlab = "Sepal Width", 
  ylab = "Sepal Length", bty = 'n', las = 1)
legend("topright", legend = levels(iris$Species), 
  fill = pal, bty = 'n', border = NA)

# <unlabeled code block>
library(MASS)
m <- lda(Species ~ Sepal.Width + Sepal.Length, data = iris)
p <- predict(m)
predicted_species <- p$class
table(predicted_species, iris$Species)

# codeblock1
a <- 1+1
b <- mean(a) + 10
# Dit is commentaat
c <- a+b
c
dta <- iris

# <unlabeled code block>
dta$foo <- dta$Sepal.Width/dta$Sepal.Length
dta[1:20, ]

# <unlabeled code block>
warning("FOO")
plot(dta$Sepal.Width, dta$Petal.Width)

# <unlabeled code block>
md_figure({
plot(dta$Sepal.Length, dta$Petal.Length)
}, name = "foo")

# nooutput1
a <- 1+1
a

# nooutput2
a <- 1+1
a

# dontrun
stop("Dit is een error")

#' Output filters for code blocks in markdown
#'
#' @param code character vector containing the code of the code block
#' @param language the language in which the code is written (as specified in 
#'   the markdown file).
#' @param id the identifier of the code block.
#' @param ... additional arguments specified as arguments in the code block are
#'   passed on to the filter function. Often these are ignored, or they are 
#'   passed on to other functions (see 'Details').
#' @param echo the code in \code{code} is repeated in the output. 
#' @param results include the results of running the code in the output. The 
#'   output of code that explicitly writes to standard output is always 
#'   included.
#' @param drop_empty do not include any output if the resulting code block 
#'   would be empty.
#' @param eval if FALSE do not run the code and just include the code in the
#'   output.
#' @param cmd the command to use in the \code{\link{system2}} call. Only needed
#'   when different than the language. 
#' @param comment_char string prepended to output of commands run by
#'   \code{output_shell}.
#' @param formatter function that will format the R-code and resulting output
#'   (if requested). See \code{\link{format_traditional}} for possible options.
#' @param capture_warnings include warnings in the output. 
#' @param capture_messages include messages in the output. 
#' @param muffle_warnings do not show warnings in the console.
#' @param muffle_messages do not show messages in the console.
#'
#' @details
#' The filter functions \code{output_table} and \code{output_figure} call 
#' \code{\link{md_table}} and \code{\link{md_figure}} respectively; additional
#' arguments are passed on to those functions. Other filter functions ignore the 
#' additional arguments. 
#'
#' It is also possible to write custom output filter. An output filter should have
#' \code{code}, \code{language} and \code{id} as its first three arguments. It 
#' should either return a character vector containing the markdown that should be
#' included in the resulting markdown file or an object that can be directly 
#' included in the pandoc parse tree. If the function does not return a character
#' vector it is assumed the latter is returned. \code{simplermarkdown} defines a
#' small number of valid object constructors: \code{\link{raw_block}} and
#' \code{\link{markdown_block}}.
#'
#' The custom function should be available when running the markdown document
#' through pandoc. The easiest way is to \code{\link{source}} or define the 
#' function in the markdown document before using it. 
#'
#' The filter function \code{output_shell} can be used to process chunks of code
#' from other languages than R. These chunks of code are written to a temporary
#' file which is then ran using the \code{cmd} using a call to
#' \code{\link{system2}}. The output of that is captured and when \code{results
#' = TRUE} included in the output. The output lines are prepended by
#' \code{comment_char}. When \code{echo = TRUE} the code is also included before
#' the output.
#'
#' @return
#' The functions either return a character vector with markdown,  
#' or return a list with the correct structure to include in the pandoc parse 
#' tree. 
#'
#' @rdname output_fun
#' @export
#' 
output_table <- function(code, language = "R", id = "",  ...) {
  tab <- source(exprs = str2expression(code), echo = FALSE, keep.source = TRUE)
  md_table(tab$value, as_character = TRUE, ...)
}


#' @rdname output_fun
#' @export
#' 
output_figure <- function(code, language = "R", id = "", ...) {
  md_figure(code, as_character = TRUE, id = id, ...)
}


#' @rdname output_fun
#' @export
#' 
output_eval <- function(code, language = "R", id = "", echo = TRUE, 
    results = TRUE, drop_empty = TRUE, eval = TRUE, 
    formatter = getOption("md_formatter", default = format_traditional), 
    capture_warnings = FALSE, capture_messages = results, 
    muffle_warnings = FALSE, muffle_messages = TRUE,
    ...) {
  if (eval == FALSE) return(markdown_block(code, language, id, ...))
  res <- run_and_capture(code, results = results, echo = echo, 
    capture_warnings = capture_warnings, capture_messages = capture_messages,
    muffle_warnings = muffle_warnings, muffle_messages = muffle_messages)
  res <- formatter(res)
  res <- paste0(res, collapse="\n")
  if (drop_empty) {
    # Check if we have only empty lines or no lines at all; in that case
    # return empty markdown
    empty_output <- (length(res) == 0) || all((grepl("^[[:blank:]]*$", res)))
    if (empty_output) return(raw_block(""))
  }
  markdown_block(res, language, id, ...)
}

#' @rdname output_fun
#' @export
#' 
output_raw <- function(code, language = "R", id = "", ...) {
  res <- utils::capture.output(
    source(exprs = str2expression(code), echo = FALSE, keep.source = TRUE)
  )
  res <- paste0(res, collapse="\n")
  raw_block(res)
}

#' @rdname output_fun
#' @export 
#' 
output_str <- function(code, language = "R", id = "", ...) {
  res <- source(exprs = str2expression(code), echo = FALSE, keep.source = TRUE)
  res <- paste0(as.character(res$value), collapse="\n")
  str_block(res)
}

#' @rdname output_fun
#' @export 
#' 
output_shell <- function(code, language, id = "", cmd = language,
    echo = TRUE, results = TRUE, comment_char = "# ", ...) {
  fn <- tempfile()
  writeLines(code, fn)
  on.exit(file.remove(fn))
  res <- system2(cmd, fn, stdout = TRUE, stderr = TRUE)
  input <- if (echo) paste0(code, collapse = "\n") else NULL
  output <- if (results) paste0("# ", res, collapse = "\n") else NULL
  markdown_block(paste0(c(input, output), collapse = "\n\n"), language, ...)
}

# Wrappers around the mdweave_to_* functions and mdtangle function intended
# for use for handling vignettes. The main difference is that these functions
# check if pandoc is available and if not will generate dummy output and not
# generate an error.

mdweave_to_html_vignette <- function(fn, ofn = file_subs_ext(basename(fn), "html", FALSE), ...) {
  if (!has_pandoc()) {
    writeLines("Cannot find pandoc. Not able to weave vignette.", ofn)
    ofn
  } else {
    mdweave_to_html(fn, ofn, ...)
  }
}


mdweave_to_pdf_vignette <- function(fn, ofn = file_subs_ext(basename(fn), "pdf", FALSE), ...) {
  if (!has_pandoc()) {
    writeLines("Cannot find pandoc. Not able to weave vignette.", ofn)
    ofn
  } else {
    mdweave_to_pdf(fn, ofn, ...)
  }
}


mdtangle_vignette <- function(fn, ofn = file_subs_ext(basename(fn), ".R"), ...) {
  if (!has_pandoc()) {
    writeLines("# Cannot find pandoc. Not able to tangle vignette.", ofn)
    ofn
  } else {
    mdtangle(fn, ofn, ...)
  }
}

# mdweave a couple of documents and check if the output is the
# same as the corresponding reference documents

library(simplermarkdown)

if (simplermarkdown:::has_pandoc()) {

  # We only want to check if the output of mdweave matches the reference if the
  # pandoc version matches that which generated the reference. Otherwise there
  # is a large chance that the output does not match
  pandoc_version <- system("pandoc --version", intern = TRUE)
  #correct_pandoc_version <- c("pandoc 2.5",
  #  "Compiled with pandoc-types 1.17.5.4, texmath 0.11.2.2, skylighting 0.7.7")
  correct_pandoc_version <- "pandoc 2.17.1.1"
  is_correct_pandoc_version <- all(pandoc_version[1] == correct_pandoc_version)


  # Create tempdir
  dir <- tempdir()
  dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  oldwd <- setwd(dir)

  # When running tests the environment variable R_TESTS is set. This
  # causes issues when running a second R instance as mdweave does. Therefore
  # we unser it.
  old_env <- Sys.getenv("R_TESTS")
  Sys.setenv("R_TESTS" = "")

  # =============================================================================
  message("Checking iris.md")
  md <- "iris.md"

  message("Weave file")
  fn <- system.file(file.path("examples", md), package = "simplermarkdown")
  mdweave(fn)

  message("Check if result generated")
  stopifnot(file.exists(md))

  if (is_correct_pandoc_version) {
    message("Compare to reference")
    lines <- readLines(md)
    fn_ref <- system.file(file.path("examples_output", md), 
      package = "simplermarkdown")
    lines_ref <- readLines(fn_ref)
    print(all.equal(lines, lines_ref))
    system(paste0("diff ", md, " ", fn_ref))
    writeLines(lines)
    stopifnot(isTRUE(all.equal(lines, lines_ref)))
  }

  message("Check images")
  # We don't check if the images are exactly the same; this probably 
  # depends on the exact output device used; versions etc.
  stopifnot(file.exists("figures/iris.png"))

  message("Cleanup")
  #unlink(md)
  unlink("figures")

  # =============================================================================
  message("Checking iris.md with format_copypaste")
  md <- "iris_copypaste.md"

  message("Weave file")
  fn <- system.file(file.path("examples", md), package = "simplermarkdown")
  mdweave(fn)

  message("Check if result generated")
  stopifnot(file.exists(md))

  if (is_correct_pandoc_version) {
    message("Compare to reference")
    lines <- readLines(md)
    fn_ref <- system.file(file.path("examples_output", md), 
      package = "simplermarkdown")
    lines_ref <- readLines(fn_ref)
    print(all.equal(lines, lines_ref))
    system(paste0("diff ", md, " ", fn_ref))
    writeLines(lines)
    stopifnot(isTRUE(all.equal(lines, lines_ref)))
  }

  message("Check images")
  # We don't check if the images are exactly the same; this probably 
  # depends on the exact output device used; versions etc.
  stopifnot(file.exists("figures/iris.png"))

  message("Cleanup")
  #unlink(md)
  unlink("figures")

  # =============================================================================
  message("Checking example1.md")
  md <- "example1.md"

  message("Weave file")
  fn <- system.file(file.path("examples", md), package = "simplermarkdown")
  mdweave(fn)

  message("Check if result generated")
  stopifnot(file.exists(md))

  if (is_correct_pandoc_version) {
    message("Compare to reference")
    lines <- readLines(md)
    fn_ref <- system.file(file.path("examples_output", md), 
      package = "simplermarkdown")
    lines_ref <- readLines(fn_ref)
    print(all.equal(lines, lines_ref))
    system(paste0("diff ", md, " ", fn_ref))
    stopifnot(isTRUE(all.equal(lines, lines_ref)))
  }

  message("Check images")
  # We don't check if the images are exactly the same; this probably 
  # depends on the exact output device used; versions etc.
  stopifnot(file.exists("figures/foo.png"))
  stopifnot(file.exists("figures/test.pdf"))

  message("Cleanup")
  unlink(md)
  unlink("figures")


  # =============================================================================
  # An markdown file without a header block gave an error with version 0.0.3. 
  # Regression test
  fn <- tempfile(fileext=".md")
  writeLines("# Introduction", fn)
  fn2 <- tempfile(fileext=".md")
  mdweave(fn, fn2)
  unlink(fn)
  unlink(fn2)

  
  # =============================================================================
  # END
  Sys.setenv("R_TESTS" = old_env)
  setwd(oldwd)
}

#' Run the code in a markdown file and generate a new markdown file
#'
#' @param fn filename of the markdown file (should use pandoc markdown).
#' @param ofn name of the resulting markdown file.
#' @param cmd1 command used to run pandoc. See details. 
#' @param cmd2 command used to run pandoc. See details. 
#' @param ... ignored
#'
#' @details
#' \code{mdweave} calls pandoc twice. In the first call the markdown file is
#' parsed by pandoc and the parse tree is written to a temporary file. This
#' parse tree is the read by mdweave and any R-code in the tree is executed
#' resulting in a modified parse tree. This file is then stored to a new
#' temporary file. Pandoc is the called a second time to convert the new
#' parse tree to a markdown file. 
#'
#' The arguments \code{cmd1} and \code{cmd2} contain the calls used to run 
#' pandoc. The arguments can be used to, for example pas additional arguments
#' to pandoc. They use positional arguments. In \code{cmd1}, the first argument
#' (\code{%1$s}) is the input file name and the second (\code{%2$s}) the 
#' temporary file containing the parsed tree. In \code{cmd2}, the first argument
#' is the temporary file with the modified parse tree and the second argument
#' the output file.
#'
#' \code{cmd2} also has a third argument (\code{%3$s}) that contains a list of
#' extensions that are enabled or disabled. This is because some extensions
#' interfere with the conversion of the parsed tree to markdown. See
#' \code{\link{get_extensions}} to see which extensions are disabled.
#'
#' @return
#' Returns the file name of the file generated (\code{ofn}). Called mainly for the
#' side effect of parsing and generating a markdown file (and possibly secondary
#' files such as figures).
#'
#' @export
#' 
mdweave <- function(fn, ofn = file_subs_ext(basename(fn), ".md", FALSE), 
    cmd1 = 'pandoc -s "%1$s" -t json -o "%2$s"', 
    cmd2 = 'pandoc -s "%1$s" -t markdown%3$s -o "%2$s"', ...) {
  # Check if output filename does nog conflict with input filename
  if (ofn == fn) stop("Output file (ofn) would overwrite input file (fn). ", 
      "Specify another output filename (ofn).")
  # Convert markdown to a parse tree in json
  tmp_ofn <- tempfile(fileext = ".json")
  cmd1 <- sprintf(cmd1, fn, tmp_ofn)
  run_cmd(cmd1, paste0("Failed to parse the markdown document to JSON (cmd1). ",  
    "This probably means that the input markdown document contains errors. ", 
    "The command was '%3$s'." ))
  # Set environment variable with location of output file
  # Possibly needed for filters to know where to put output
  # files such as figures
  outputdir <- dirname(ofn)
  dir.create(outputdir, recursive = TRUE, showWarnings = FALSE)
  Sys.setenv(MDOUTDIR = outputdir)
  on.exit(Sys.unsetenv("MDOUTDIR"))
  # Filter json (run R code in json) and save result
  dta <- filter_pandoc_json_tree(tmp_ofn)
  tmp_ifn <- tempfile(fileext = ".json")
  writeLines(rjson::toJSON(dta), tmp_ifn)
  # Convert json back to markdown
  extensions <- get_extensions()
  cmd2 <- sprintf(cmd2, tmp_ifn, ofn, extensions)
  run_cmd(cmd2, paste0("Failed to convert the processed JSON file back to ", 
    "markdown (cmd2). Either on of the output functions contains an error ",
    "or simplermarkdown has written invalid JSON. ",
    "The command was '%3$s'." ))
  # Cleanup; an earlier version used on.exit for this; currrent solution has the 
  # advantage that in case of an error the old file are not deleted and can be
  # used for debugging
  file.remove(tmp_ofn)
  file.remove(tmp_ifn)
  # Return filename of the final file 
  invisible(ofn)
}

run_cmd <- function(cmd, error_msg = "Failed to run '%1$s'; failed with status %2$d") {
  command <- gsub("^([a-zA-Z0-9]+) (.*)$", "\\1", cmd)
  args <- gsub("^([a-zA-Z0-9]+) (.*)$", "\\2", cmd)
  suppressWarnings({
    status <- system2(command, args)
  })
  if (status != 0) {
    if (status == 127) {
      stop("Running '", command, "' had status 127; this generally means that ",
        "'", command, "' could not be found. Make sure '", command, 
        "' in on the path.")
    } else {
      stop(suppressWarnings(sprintf(error_msg, command, status, cmd)))
    }
  }
}

.onLoad <- function(libname, pkgname) {
  tools::vignetteEngine("mdweave_to_html", 
    weave = mdweave_to_html_vignette, 
    tangle = mdtangle_vignette,
    pattern = "[.][R]?md$", package = "simplermarkdown")
  tools::vignetteEngine("mdweave_to_pdf", 
    weave = mdweave_to_pdf_vignette, 
    tangle = mdtangle_vignette,
    pattern = "[.][R]?md$", package = "simplermarkdown")
}

# mdtangle a couple of documents and check if the output is the
# same as the corresponding reference documents

library(simplermarkdown)

if (simplermarkdown:::has_pandoc()) {

  # Create tempdir
  dir <- tempdir()
  dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  oldwd <- setwd(dir)

  # When running tests the environment variable R_TESTS is set. This
  # causes issues when running a second R instance as mdweave does. Therefore
  # we unser it.
  old_env <- Sys.getenv("R_TESTS")
  Sys.setenv("R_TESTS" = "")


  # =============================================================================
  message("Checking iris.md")
  md <- "iris.md"
  r <- "iris.R"

  message("Tangle file")
  fn <- system.file(file.path("examples", md), package = "simplermarkdown")
  mdtangle(fn)

  message("Compare to reference")
  lines <- readLines(r)
  fn_ref <- system.file(file.path("examples_output", r), 
    package = "simplermarkdown")
  lines_ref <- readLines(fn_ref)
  stopifnot(isTRUE(all.equal(lines, lines_ref)))

  message("Cleanup")
  unlink(r)

  # =============================================================================
  message("Checking example1.md")
  md <- "example1.md"
  r <- "example1.R"

  message("Tangle file")
  fn <- system.file(file.path("examples", md), package = "simplermarkdown")
  mdtangle(fn)

  message("Compare to reference")
  lines <- readLines(r)
  fn_ref <- system.file(file.path("examples_output", r), 
    package = "simplermarkdown")
  lines_ref <- readLines(fn_ref)
  stopifnot(isTRUE(all.equal(lines, lines_ref)))

  message("Cleanup")
  unlink(r)


  Sys.setenv("R_TESTS" = old_env)
  setwd(oldwd)

}

library(simplermarkdown)

# -----------------------------------------------------------------------------
src <- "a <- 1"

r <- run_and_capture(src)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 1)
stopifnot(isTRUE(all.equal(r[[1]]$input, src)))
stopifnot(length(r[[1]]$output) == 0)

r <- run_and_capture(src, echo = FALSE)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 0)

# -----------------------------------------------------------------------------
src <- "a <- 1\na"

r <- run_and_capture(src)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 2)
stopifnot(isTRUE(all.equal(r[[1]]$input, "a <- 1")))
stopifnot(length(r[[1]]$output) == 0)
stopifnot(isTRUE(all.equal(r[[2]]$input, "a")))
stopifnot(isTRUE(all.equal(r[[2]]$output, "[1] 1")))

r <- run_and_capture(src, echo = FALSE)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 1)
stopifnot(isTRUE(all.equal(r[[1]]$input, character(0))))
stopifnot(isTRUE(all.equal(r[[1]]$output, "[1] 1")))

r <- run_and_capture(src, results = FALSE)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 2)
stopifnot(isTRUE(all.equal(r[[1]]$input, "a <- 1")))
stopifnot(length(r[[1]]$output) == 0)
stopifnot(isTRUE(all.equal(r[[2]]$input, "a")))
stopifnot(length(r[[2]]$output) == 0)

r <- run_and_capture(src, echo = FALSE, results = FALSE)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(length(r) == 0)
stopifnot(is.list(r))

# -----------------------------------------------------------------------------
src <- ""
r <- run_and_capture(src)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(is.list(r) && length(r) == 0)

# -----------------------------------------------------------------------------
src <- character(0)
r <- run_and_capture(src)
stopifnot(isTRUE(all.equal(attr(r, "code"), src)))
stopifnot(is.list(r) && length(r) == 0)

# Thin wrapper around capture.output that also captures warnings and messages
capture_output <- function(expr, capture_warnings = TRUE, capture_messages = TRUE, 
    muffle_warnings = FALSE, muffle_messages = TRUE) {
  utils::capture.output({
    base::withCallingHandlers({
      expr
      }, warning = function(e) {
        if (capture_warnings) cat("Warning: ", conditionMessage(e), "\n")
        if (muffle_warnings) invokeRestart("muffleWarning")
      }, message = function(e) {
        if (capture_messages) cat(e$message, "\n")
        if (muffle_messages) invokeRestart("muffleMessage")
      }
    )
  })
}

#' Extract code from the code blocks in a markdown file
#'
#' @param fn filename of the markdown file (should use pandoc markdown).
#' @param ofn name of the resulting R-script
#' @param extra_arguments extra arguments passed on to pandoc. Should be a length 1 
#'  character vector.
#' @param cmd command used to run pandoc. See details. 
#' @param ... ignored
#'
#' @details
#' \code{mdtangle} calls pandoc. Pandoc will parse the markdown document and
#' write the parsed file to temporary file. This file is read by 
#' \code{mdtangle} and the code is extracted from it and written to \code{ofn}.
#'
#' Using the \code{cmd} argument the exact command used to run pandoc can be
#' modified. It is passed on to \code{\link{sprintf}} and uses positional 
#' arguments: (1) name of the input file, (2) location of the temporary file
#' to which the parsed document is written, (3) the value of 
#' \code{extra_arguments}.
#'
#' @return 
#' Returns the filename of the generated file. 
#'
#' @export
#' 
mdtangle <- function(fn, ofn = file_subs_ext(basename(fn), ".R"), 
    extra_arguments = "", cmd = 'pandoc %3$s -s "%1$s" -t json -o "%2$s"', ...) {
  tmp_ofn <- tempfile(fileext = ".json")
  on.exit(file.remove(tmp_ofn))
  cmd <- sprintf(cmd, fn, tmp_ofn, extra_arguments)
  system(cmd)
  dta <- rjson::fromJSON(file = tmp_ofn, simplify = FALSE)
  default_fun <- "output_eval"
  verbosity   <- 1
  code <- character(0)
  gather_code <- function(block, default_fun = "eval", verbosity = 1) {
    b <- get_block(block)
    if (!is.null(b) && b$language == "R") {
      id <- if (b$id == "") "<unlabeled code block>" else b$id
      if (verbosity > 0) message("Tangling code in block '", id, "'.")
      code <<- c(code, paste("#", id), b$code, "")
    }
    block
  }
  ignore_inline <- function(block, verbosity = 1)  {
    block
  }
  # Go over all of the blocks in the tree; check if they contain R code and
  # evaluate the code
  dta$blocks <- parse_blocks(dta$blocks, default_fun = default_fun, 
    verbosity = verbosity, eval_block = gather_code, eval_inline = ignore_inline)
  writeLines(code, con = ofn)
  invisible(ofn)
}

#' Return a code block object that can be included in the pandoc parse tree
#'
#' @param content a character vector containing the code
#' @param language language of the code in the code block
#' @param id optional id of the code block
#' @param ... additional arguments should be named. These are added to the
#'   markdown block as additional arguments.
#'
#' @return 
#' Returns a \code{list} with the correct structure for a code block in 
#' the pandoc parse tree.
#'
#' @export
markdown_block <- function(content, language, id = "", ...) {
  # parse extra arguments
  # This should be an unnamed list with each element a 2 element
  # vector; first element the name; second element the value
  extra <- list(...)
  if (!all(names(extra) != ""))
    stop("Additional arguments all need to be named")
  extra <- lapply(extra, \(x) as.character(x)[1])
  extra <- lapply(names(extra), \(x) list(x, extra[[x]]))
  # block
  list(
    t = "CodeBlock",
    c = list(
      list(
        id,
        list(language),
        extra
      ),
      content
    )
  )
}

#' Return a raw chunk of text that can be included in the pandoc parse tree
#'
#' @param content a character vector containing the content to include
#'   in the final document.
#' @param language language of the content 
#'
#' @details
#' A raw block is included as is into the final markdown document. This can
#' be used for example to include raw chunks of markdown.
#'
#' @return 
#' Returns a \code{list} with the correct structure for a \code{RawBlock} in 
#' the pandoc parse tree.
#'
#' @export
raw_block <- function(content, language = "markdown") {
  list(
    t = "RawBlock", 
    c = list(language, paste0(content, collapse="\n"))
  )
}

get_block <- function(block) {
  if (block$t != "CodeBlock" && block$t != "Code") return(NULL)
  id <- block$c[[1]][[1]]
  language <- if (length(block$c[[1]][[2]]))
    block$c[[1]][[2]][[1]] else ""
  #if (language != "R") return(NULL)
  args1 <- if (length(block$c[[1]][[2]]) > 1) {
    utils::tail(unlist(block$c[[1]][[2]]), -1)
  } else character(0L)
  arguments <- get_block_arguments(block$c[[1]][[3]])
  code <- block$c[[2]]
  list(
    id = id,
    language = language,
    arguments_single = args1,
    arguments = arguments,
    code = code
  )
}

get_block_arguments <- function(arguments) {
  sapply(arguments, function(a) {
    val <- a[[2]]
    if (val == "TRUE") { 
      val <- TRUE 
    } else if (val == "FALSE") {
      val <- FALSE
    } else if (grepl("^[0-9]+$", val)) {
      val <- as.numeric(val)
    }
    res <- list(val)
    names(res) <- a[[1]]
    res
  })
}

#' Generate a figure and generate the markdown to include the figure
#'
#' Will evaluate the expressions in \code{expr} and capture the output on the
#' given plotting device in the given file. It will then generate the markdown
#' needed to include that figure in a markdown document.
#'
#' @param expr the expressions to evaluate. Will generally contain plotting 
#'   commands. The expressions are evaluated in the global environment.
#' @param name the name of the figure. 
#' @param caption text of the caption. When omitted no caption is added to the
#'   figure.
#' @param id id of the figure. When omitted or equal to NULL or an empty 
#'   character, no id is added to the figure.
#' @param dir name of the directory in which to store the file. 
#' @param device the graphics device to use for creating the image.
#' @param ... passed on to the graphics device. 
#' @param as_character return the figure as a character vector. If \code{FALSE}
#'   the figure will be written to the standard output. 
#' @param echo the code in \code{code} is repeated in the output. 
#' @param results include the results of running the code in the output. The 
#'   output of code that explicitly writes to standard output is always 
#'   included.
#' @param formatter function that will format the R-code and resulting output
#'   (if requested). See \code{\link{format_traditional}} for possible options.
#' @param capture_warnings include warnings in the output. 
#' @param capture_messages include messages in the output. 
#' @param muffle_warnings do not show warnings in the console.
#' @param muffle_messages do not show messages in the console.
#'
#' @details
#' The image is stored in the file \code{dir/name.device}. 
#'
#' @return
#' When \code{as_character = FALSE} a character vector with the markdown needed
#' to include the generated figure in a markdown file is returned. Otherwise, 
#' nothing is returned; the markdown is written to the console.
#' 
#' @export
#' 
md_figure <- function(expr, name, caption = "", id = "",
    dir = file.path(Sys.getenv("MDOUTDIR", "."), "figures"), 
    device = c("png", "pdf"), ...,
    as_character = FALSE, echo = FALSE, results = FALSE,
    formatter = getOption("md_formatter", default = format_traditional), 
    capture_warnings = FALSE, capture_messages = results, 
    muffle_warnings = FALSE, muffle_messages = TRUE) {
  dir.create(dir, recursive = TRUE, showWarnings = FALSE)
  device <- match.arg(device)
  extension <- ""
  if (device == "png") { 
    fn <- file.path(dir, paste0(name, ".png"))
    grDevices::png(fn, ...)
  } else if (device == "pdf") { 
    fn <- file.path(dir, paste0(name, ".pdf"))
    grDevices::pdf(fn, ...)
  }
  on.exit(grDevices::dev.off())
  # in order to handle both md_figure(plot(1:10) and 
  # md_figure(str2expression("plot(1:10)"))
  expr <- if (is.expression(expr) || is.character(expr)) expr else 
    as.expression(substitute(expr))
  res <- run_and_capture(expr, results = results, echo = echo, 
    capture_warnings = capture_warnings, capture_messages = capture_messages,
    muffle_warnings = muffle_warnings, muffle_messages = muffle_messages)
  res <- formatter(res)
  res <- paste0(res, collapse="\n")
  # Check if, besides a figure, we also need to add the commands and
  # other output to the output
  if (echo || results) {
    res <- paste0(c("\n```{.R}", res, "```\n"), collapse = "\n")
  } else res <- character(0)
  # Generate the markdown for the figure
  id_str <- if (!is.null(id) && id != "") 
    id_str <- paste0("{#", id, "}") else id_str = ""
  res <- paste0(res, "\n![", caption, "](", fn, ")", id_str, "\n")
  if (as_character) {
    res
  } else {
    cat(res)
  }
}

#' @importFrom rjson fromJSON
#' 
filter_pandoc_json_tree <- function(con) {
  if (missing(con)) {
    con <- file("stdin")
    input <- readLines(con, warn = FALSE)
    close(con)
    dta <- rjson::fromJSON(input, simplify = FALSE)
  } else {
    dta <- rjson::fromJSON(file = con, simplify = FALSE)
  }
  # Extra step needed to get correct output; when meta is empty make sure
  # it is a named list; otherwire toJSON will generate 'meta: []' instead
  # of 'meta: {}' what pandoc expects
  if (is.list(dta$meta) && length(dta$meta) == 0) 
    names(dta$meta) <- character(0)
  default_fun <- "output_eval"
  verbosity   <- 1
  # Go over all of the blocks in the tree; check if they contain R code and
  # evaluate the code
  dta$blocks <- parse_blocks(dta$blocks, default_fun = default_fun, 
    verbosity = verbosity)
  dta
}


parse_blocks <- function(blocks, verbosity = 1, default_fun = "output_eval", 
    eval_block = evaluate_code_block, eval_inline = evaluate_inline_code) {
  # The following type can all be handled the same way
  basic_types <- c("Emph", "Para", "Plain", "BlockQuote")
  # In the following types of content we will not look for code blocks and these
  # will nog be processed further
  types_to_ignore <- c("Str", "Space", "Str", "Strong", "Link", "Image", 
    "Table", "SoftBreak")
  # Loop over the blocks and depending on the type process further; when we have
  # a block of R-code we will evaluate the code within it
  for (i in seq_along(blocks)) {
    block <- blocks[[i]]
    if (block$t == "Code") {
      block <- eval_inline(block, verbosity = verbosity)
    } else if (block$t == "CodeBlock") {
      block <- eval_block(block, verbosity = verbosity, 
        default_fun = default_fun)
    } else if (block$t == "Header") {
      block$c[[3]] <- parse_blocks(block$c[[3]], verbosity, default_fun,
        eval_block, eval_inline)
    } else if (block$t == "BulletList") {
      for (j in seq_along(block$c)) 
        block$c[[j]] <- parse_blocks(block$c[[j]], verbosity, default_fun,
        eval_block, eval_inline)
    } else if (block$t == "OrderedList") {
      for (j in seq_along(block$c[[2]])) 
        block$c[[2]][[j]] <- parse_blocks(block$c[[2]][[j]], verbosity, default_fun,
        eval_block, eval_inline)
    } else if (block$t == "Div") {
      block$c[[2]] <- parse_blocks(block$c[[2]], verbosity, default_fun,
        eval_block, eval_inline)
    } else if (block$t %in% basic_types) {
      block$c <- parse_blocks(block$c, verbosity, default_fun,
        eval_block, eval_inline)
    } else if (block$t %in% types_to_ignore) {
      # do nothing
    } else {
      # also do nothing but report
      if (verbosity > 1)
        warning("Ignoring unsupported block type '", block$t, "'.");
    }
    blocks[[i]] <- block
  }
  blocks
}

evaluate_inline_code <- function(block, verbosity = 1) {
  b <- get_block(block)
  if (!is.null(b) && b$language == "R") {
    id <- if (b$id == "") "<unlabeled inline block>" else b$id
    if (verbosity > 0) message("Evaluating code in inline block '", id, "'.")
    block <- do.call(output_str, list(code = b$code, id = b$id, language = b$language))
  }
  block
}

evaluate_code_block <- function(block, default_fun = "output_eval", verbosity = 1) {
  b <- get_block(block)
  if (is.null(b)) return(block)

  if (b$language == "R") {
    id <- if (b$id == "") "<unlabeled code block>" else b$id
    if (verbosity > 0) message("Evaluating code in block '", id, "'.")
    if (exists("fun", b$arguments)) {
      fun <- b$arguments$fun 
      b$arguments$fun <- NULL
    } else {
      fun <- default_fun
    }
    res <- do.call(fun, c(
      list(code = b$code, id = b$id, language = b$language), b$arguments))
    block <- if (is.character(res)) raw_block(res) else res
  } else {
    # Other languages than R; only run if output fun is given
    if (exists("fun", b$arguments)) {
      id <- if (b$id == "") "<unlabeled code block>" else b$id
      if (verbosity > 0) message("Evaluating ", b$language, " code in block '", id, "'.")
      fun <- b$arguments$fun 
      b$arguments$fun <- NULL
      res <- do.call(fun, c(
        list(code = b$code, id = b$id, language = b$language), b$arguments))
      block <- if (is.character(res)) raw_block(res) else res
    }
  }
  block
}

#' Get the extensions that are needed when converting from json to markdown by 
#' pandoc.
#'
#' @details
#' Checks which extensions are available in pandoc and returns a character
#' string disabling extensions that interfere with the conversion of the
#' processed document (in json format) back to markdown. At the moment only one
#' extension is disabled.
#'
#' \describe{
#'   \item{\code{raw_attribute}}{ This extension is disabled. Some of the output
#'   functions write raw markdown blocks. This extension only recognises html
#'   and tex raw blocks. Raw markdown blocks are ignored when converting to
#'   other formats. }
#' }
#'
#' @return 
#' Returns a character vector of the form \code{"+extension1-extension2"} with
#' the extensions that are to be enabled (\code{+}) or disables (\code{-}).
#' The function calls \code{pandoc} to check which extensions are available.
#'
#' @export
get_extensions <- function() {
  extensions <- character(0)
  # Get the list of extensions supported by the installed pandoc
  supported_extensions <- system("pandoc --list-extensions", intern = TRUE)
  # when converting from json to markdown we need to disable the raw_attribute
  # extension. Some of the output functions write raw markdown. With this
  # extension this raw markdown is put into a
  # ```{=markdown}
  # markdown
  # ```
  # block. This is not recognised when converting to other formats.
  has_raw <- any(grepl("^[+-]raw_attribute", supported_extensions))
  if (has_raw) {
    extensions <- paste0(extensions, "-raw_attribute")
  }
  extensions
}

#' Replace file extension by another file extension
#'
#' @param fn character vector with file names
#' @param new_ext character vector of length one with the new extension (when
#'   it does not start with a period a period is added).
#' @param check check if the new file name is not equal to the original 
#'   filename. If so, generate an error.
#' 
#' @return 
#' Returns a character vector of the same length of \code{fn} with the
#' extension of the file names in \code{fn} replaced by \code{new_ext}.
#'
#' @export
file_subs_ext <- function(fn, new_ext, check = TRUE) {
  if (substr(new_ext, 1, 1) != ".") new_ext <- paste0(".", new_ext)
  newfn <- paste0(tools::file_path_sans_ext(fn), new_ext);
  if (check && any(newfn == fn))
    stop("fn already has extension '", new_ext, "'.")
  newfn
}

library(simplermarkdown)

expect_equal <- function(a, b) {
  stopifnot(isTRUE(all.equal(a,b)))
}

expect_equal(
  file_subs_ext("foo.x", "y"), 
  "foo.y")

expect_equal(
  file_subs_ext(c("foo.x", "bar.z"), "y"), 
  c("foo.y", "bar.y"))

expect_equal(
  file_subs_ext(c(".x", "foo/.bar.z"), "y"), 
  c(".x.y", "foo/.bar.y"))

expect_equal(
  file_subs_ext("", "y"), 
  ".y")

#' Run the code in a markdown file and generate a new document
#' 
#' @param fn filename of the markdown file (should use pandoc markdown).
#' @param ofn name of the resulting file.
#' @param extra_arguments2 extra arguments passed on to pandoc. Should be a length 1 
#'  character vector.
#' @param run_in_temp When TRUE the intermediary markdown file and generated figures (when
#'  not using custom paths) are created in a temporary directory. Otherwise these will be
#'  generated in the same directory as the output file.
#' @param cmd2 command used to run pandoc. See details. 
#' @param ... additional arguments are passed on to \code{\link{mdweave}}.
#'
#' @details
#' These functions first call \code{\link{mdweave}} to run the code in the original
#' file and convert the original markdown file to a new markdown file. This second
#' markdown file is then converted to the desired output format using a second run
#' of \code{pandoc}.
#'
#' In case of converting to pdf the file is required to have the extension 
#' \code{.pdf}. In case of converting to LaTeX, the file cannot have the extension 
#' \code{.pdf}. That is because in both cases the file is first converted to 
#' LaTeX. In case of a file with the extension \code{.pdf} the file is than 
#' further converted to PDF. 
#'
#' @return
#' Returns the name of the resulting outout file.
#'
#' @export
#' @rdname mdweave_to
mdweave_to_pdf <- function(fn, ofn = file_subs_ext(basename(fn), ".pdf", FALSE), 
    extra_arguments2 = "--self-contained", run_in_temp = TRUE,
    cmd2 = 'pandoc %3$s -s "%1$s" -t latex -o "%2$s"', ...) {
  # Check if extension of ofn is .pdf; this is required by pandoc
  if (!grepl("\\.pdf$", ofn)) stop("ofn should have the extension .pdf.")
  # First convert fn to md using mdweave
  if (run_in_temp) {
    dir <- tempfile()
    dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    ofn_md <- file.path(dir, file_subs_ext(basename(fn), ".md", FALSE))
  } else {
    ofn_md <- file_subs_ext(ofn, ".md", FALSE)
    if (ofn_md == fn) ofn_md <- paste0(ofn, ".md")
  }
  mdweave(fn, ofn_md, ...)
  # Convert md to pdf
  if (ofn == fn) stop("Output file (ofn) would overwrite input file (fn). ", 
      "Specify another output filename (ofn).")
  cmd2 <- sprintf(cmd2, ofn_md, ofn, extra_arguments2)
  run_cmd(cmd2, paste0("Failed to convert the markdown file to PDF. ", 
    "You may want to first run mdweave and inspect the resulting markdown ", 
    "file. The command was '%3$s'."))
  invisible(ofn)
}

#' @export
#' @rdname mdweave_to
mdweave_to_tex <- function(fn, ofn = file_subs_ext(basename(fn), ".tex", FALSE), 
    extra_arguments2 = "--self-contained", run_in_temp = TRUE,
    cmd2 = 'pandoc %3$s -s "%1$s" -t latex -o "%2$s"', ...) {
  if (grepl("\\.pdf$", ofn)) stop("ofn cannot have the extension .pdf.")
  # First convert fn to md using mdweave
  if (run_in_temp) {
    dir <- tempfile()
    dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    ofn_md <- file.path(dir, file_subs_ext(basename(fn), ".md", FALSE))
  } else {
    ofn_md <- file_subs_ext(ofn, ".md", FALSE)
    if (ofn_md == fn) ofn_md <- paste0(ofn, ".md")
  }
  mdweave(fn, ofn_md, ...)
  # Convert md to pdf
  if (ofn == fn) stop("Output file (ofn) would overwrite input file (fn). ", 
      "Specify another output filename (ofn).")
  cmd2 <- sprintf(cmd2, ofn_md, ofn, extra_arguments2)
  run_cmd(cmd2, paste0("Failed to convert the markdown file to tex. ", 
    "You may want to first run mdweave and inspect the resulting markdown ", 
    "file. The command was '%3$s'."))
  invisible(ofn)
}


#' @export
#' @rdname mdweave_to
mdweave_to_html <- function(fn, ofn = file_subs_ext(basename(fn), ".html", FALSE), 
    extra_arguments2 = "--self-contained", run_in_temp = TRUE,
    cmd2 = 'pandoc %3$s -s "%1$s" -t html -o "%2$s"', ...) {
  # First convert fn to md using mdweave
  if (run_in_temp) {
    dir <- tempfile()
    dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    ofn_md <- file.path(dir, file_subs_ext(basename(fn), ".md", FALSE))
  } else {
    ofn_md <- file_subs_ext(ofn, ".md", FALSE)
    if (ofn_md == fn) ofn_md <- paste0(ofn, ".md")
  }
  mdweave(fn, ofn_md, ...)
  # Convert md to pdf
  if (ofn == fn) stop("Output file (ofn) would overwrite input file (fn). ", 
      "Specify another output filename (ofn).")
  cmd2 <- sprintf(cmd2, ofn_md, ofn, extra_arguments2)
  run_cmd(cmd2, paste0("Failed to convert the markdown file to HTML. ", 
    "You may want to first run mdweave and inspect the resulting markdown ", 
    "file. The command was '%3$s'."))
  invisible(ofn)
}

#' Run code and capture the output
#'
#' @param code character vector or expression with the code to run
#' @param echo the code in \code{code} is repeated in the output. 
#' @param results include the results of running the code in the output. 
#' @param output include output that is explicitly written to the output, for 
#'   example using \code{print} statements.
#' @param capture_warnings include warnings in the output. 
#' @param capture_messages include messages in the output. 
#' @param muffle_warnings do not show warnings in the console.
#' @param muffle_messages do not show messages in the console.
#'
#' @return
#' Returns a list. Each item of the list contains a list with elements
#' \code{input} and \code{output}. \code{input} contains the command/code and
#' \code{output} the corresponding output. These are empty vectors when there is
#' no output or when input and output are suppressed using one of the
#' \code{echo}/\code{results}/\code{output} statements.
#'
#' @export
#' 
run_and_capture <- function(code, echo = TRUE, results = TRUE, output = results, 
    capture_warnings = FALSE, capture_messages = results, muffle_warnings = FALSE, 
    muffle_messages = TRUE) {
  # Setup for running the code
  prompts <- list(prompt = "simplermarkdown1", 
      continue = "simplermarkdown2")
  # Run the code and capture the output
  if (is.character(code)) {
    outp <- capture_output(
      source(textConnection(code), echo = echo, spaced = FALSE,
        print.eval = results, max.deparse.length = Inf, 
        prompt.echo = prompts$prompt,
        continue.echo = prompts$continue,
        keep.source = TRUE),
      capture_warnings = capture_warnings, capture_messages = capture_messages,
      muffle_warnings = muffle_warnings, muffle_messages = muffle_messages
    )
  } else if (is.expression(code)) {
    outp <- capture_output(
      source(exprs = code, echo = echo, spaced = FALSE,
        print.eval = results, max.deparse.length = Inf, 
        prompt.echo = prompts$prompt,
        continue.echo = prompts$continue,
        keep.source = TRUE),
      capture_warnings = capture_warnings, capture_messages = capture_messages,
      muffle_warnings = muffle_warnings, muffle_messages = muffle_messages
    )
  }
  # Convert the output to something structured
  # Remove empty first line
  if (length(outp) && outp[1] == "") outp <- utils::tail(outp, -1)
  # Split into separate commands with its output
  regexp <- paste0("^", prompts$prompt)
  outp <- split(outp, cumsum(grepl(regexp, outp)))
  # For each separate command convert to a list with the command and
  # output separated.
  res <- lapply(outp, function(o) {
    regexp <- paste0("^(", prompts$prompt, "|", 
      prompts$continue, ")")
    commands <- grepl(regexp, o)
    input <- gsub(regexp, "", o[commands])
    list(input = input, output = o[!commands])
  })
  # When there are explicit print statements these are still included 
  # in the output; remove
  if (!output) {
    res <- lapply(res, function(o) {
      o$output <- character(0)
      o
    })
  }
  structure(res, code = code)
}

#' Format the result of running a block of code
#'
#' @param x the result of running the code. See 'details' for the format.
#'
#' @details
#' The input vector \code{x} should be a list. Each element of this list should
#' be a list with two elements: \code{input} and \code{output}. \code{input}
#' should contain the evaluated expression; this can be more than one line.
#' \code{output} should contain the output of the evaluation. When there is no
#' output this is character vector of length 0.
#'
#' @return
#' A character vector of length 1 with the formatted code.
#' 
#' @export
#' @rdname format
#'
format_traditional <- function(x) {
  res <- lapply(x, function(o) {
    if (length(o$input)) {
      f <- rep("+ ", length(o$input))
      f[1] <- "> "
      input <- paste0(f, o$input)
    } else input <- character(0)
    output <- o$output
    c(input, output)
  }) 
  paste0(unlist(res), collapse="\n")
}


#' @export
#' @rdname format
#'
format_copypaste <- function(x) {
  res <- lapply(x, function(o) {
    input <- if (length(o$input)) o$input else character(0)
    output <- if (length(o$output))  
      output <- paste0("## ", o$output) else character(0)
    c(input, output)
  }) 
  paste0(unlist(res), collapse="\n")
}

#' Generate a markdown table from a data.frame
#'
#' @param tab a data frame
#' @param caption text of the caption. When omitted no caption is added to the
#'   table.
#' @param as_character return the table as a character vector. If \code{FALSE}
#'   the table will be written to the standard output. 
#' @param ... unused. 
#'
#' @return
#' Then \code{as_character = FALSE} a character vector with the markdown
#' containing the table is returned.  Otherwise, 
#' nothing is returned; the markdown is then written to the console.
#'
#' @export
#' 
md_table <- function(tab, caption, as_character = FALSE, ...) {
  res <- vector("list", ncol(tab))
  for (i in seq_along(res)) {
    t <- format(tab[[i]])
    t <- format(c(names(tab)[i], t))
    nc <- max(nchar(t))
    line <- paste0(rep("-", nc), collapse ="")
    t <- c(t[1], line, utils::tail(t, -1))
    res[[i]] <- t
  }
  res <- do.call(paste, c("", res, "", sep = "|"))
  if (!missing(caption) && !is.null(caption)) {
    res <- c(paste0(": ", caption), "", res)
  }
  if (as_character) paste0(res, collapse="\n")  else writeLines(res)
}

---
title: An analysis of iris
---

## Introduction

From the help page of the iris data set:

> This famous (Fisher's or Anderson's) iris data set gives the
> measurements in centimeters of the variables sepal length and width
> and petal length and width, respectively, for 50 flowers from each of
> 3 species of iris. The species are *Iris setosa*, *versicolor*, and
> *virginica*.

## Descriptives

The table below shows for each of the iris species the mean value of the
colums in the data set.

: Mean values for each of the properties for each of the iris species.

|Species   |Sepal.Length|Sepal.Width|Petal.Length|Petal.Width|
|----------|------------|-----------|------------|-----------|
|setosa    |5.006       |3.428      |1.462       |0.246      |
|versicolor|5.936       |2.770      |4.260       |1.326      |
|virginica |6.588       |2.974      |5.552       |2.026      |

```{.R}
> pal <- hcl.colors(3, "Dark2")
> plot(iris$Sepal.Width, iris$Sepal.Length, pch = 20, 
+   col = pal[iris$Species], xlab = "Sepal Width", 
+   ylab = "Sepal Length", bty = 'n', las = 1)
> legend("topright", legend = levels(iris$Species), 
+   fill = pal, bty = 'n', border = NA)
```

![Relation between sepal length and width for the different iris species.](./figures/iris.png){#figure}

## Species prediction

``` R
> library(MASS)
> m <- lda(Species ~ Sepal.Width + Sepal.Length, data = iris)
> p <- predict(m)
> predicted_species <- p$class
> table(predicted_species, iris$Species)
                 
predicted_species setosa versicolor virginica
       setosa         49          0         0
       versicolor      1         36        15
       virginica       0         14        35
```

This model predicts in 80% of the cases the correct species. However,
this is mainly for *setosa* for the other species the model predicts the
correct species only for 71% of the records.

---
title: Title
---


# Header

- List item 
- Another list item

And some text with `inline code`.

``` {#codeblock1 .R echo=TRUE results=TRUE additionalargument=42}
a <- 1+1
b <- mean(a) + 10
# Dit is commentaat
c <- a+b
c
dta <- iris
```

And some text

``` {.R fun=output_table caption="Sample iris"}
dta$foo <- dta$Sepal.Width/dta$Sepal.Length
dta[1:20, ]
```

### And some text

The mean of `Sepal.Width` is `m<-round(mean(dta$Sepal.Width), 2)`{.R}. 
This is `ifelse(m>2, "larger", "smaller")`{.R} than 2.

```
Geen R-code
```


```{.R fun=output_figure name="test" caption="My figure" device="pdf" width=8 
  height=6}
warning("FOO")
plot(dta$Sepal.Width, dta$Petal.Width)
```


```{.R fun=output_raw}
md_figure({
plot(dta$Sepal.Length, dta$Petal.Length)
}, name = "foo")
```


# Some regresion like tests

Following should result in no code block.

```{.R #nooutput1 results=FALSE echo=FALSE}
a <- 1+1
a
```

Following should result in an empty code block.

```{.R #nooutput2 results=FALSE echo=FALSE drop_empty=FALSE}
a <- 1+1
a
```

Following code should not be run

```{.R #dontrun eval=FALSE}
stop("Dit is een error")
```

---
title: An analysis of iris
---

Introduction
-------------------------

From the help page of the iris data set:

> This famous (Fisher's or Anderson's) iris data set gives the
> measurements in centimeters of the variables sepal length and
> width and petal length and width, respectively, for 50 flowers
> from each of 3 species of iris.  The species are _Iris setosa_,
> _versicolor_, and _virginica_.


Descriptives
--------------------------

The table below shows for each of the iris species the mean value of the colums in the data set. 

```{.R #table fun=output_table caption="Mean values for each of the properties for each of the iris species."}
aggregate(iris[1:4], iris["Species"], mean)
```


```{.R #figure fun=output_figure 
  caption="Relation between sepal length and width for the different iris species." 
  name="iris" height=6 width=8 units="in" res=150 echo=TRUE}
pal <- hcl.colors(3, "Dark2")
plot(iris$Sepal.Width, iris$Sepal.Length, pch = 20, 
  col = pal[iris$Species], xlab = "Sepal Width", 
  ylab = "Sepal Length", bty = 'n', las = 1)
legend("topright", legend = levels(iris$Species), 
  fill = pal, bty = 'n', border = NA)
```



Species prediction
---------------------------------

```{.R}
library(MASS)
m <- lda(Species ~ Sepal.Width + Sepal.Length, data = iris)
p <- predict(m)
predicted_species <- p$class
table(predicted_species, iris$Species)
```

This model predicts in `round(mean(predicted_species==iris$Species)*100)`{.R}% of the
cases the correct species. However, this is mainly for *setosa* for the other species the
model predicts the correct species only for
`sel<-iris$Species!="setosa";round(100*mean(predicted_species[sel] == iris$Species[sel]))`{.R}% of
the records.

<!--
%\VignetteEngine{simplermarkdown::mdweave_to_html}
%\VignetteIndexEntry{Introduction to simplermarkdown}
-->
---
title: simplermarkdown
css: "style.css"
---

Introduction to simplermarkdown
--------------------------------------

First write a markdown file using [pandoc
markdown](https://pandoc.org/MANUAL.html#pandocs-markdown).  The package uses
pandoc to parse the markdown and runs any code blocks tagged as being R-code.
The result a new markdown file that can be further processed using pandoc and
compiled to, for example, PDF or HTML.

To process your markdown file you can use the `mdweave` function:

```
mdweave("mydocument.md", "mydocument_woven.md")
```

The resulting document `mydocument_woven.md` can then be processes further using
pandoc. 

Below is a ready to run example using an example document in the package:

```
library(simplermarkdown)

example1 <- system.file("examples/example1.md", package = "simplermarkdown")

mdweave(example1, "example1_woven.md")

system("pandoc example1_woven.md -o example1.pdf")
```

The package also includes the functions `mdweave_to_pdf`, `mdweave_to_html` and
`mdweave_to_tex`, that combine these last two function calls. Therefore, the
example above could also have been written as:

```
library(simplermarkdown)
example1 <- system.file("examples/example1.md", package = "simplermarkdown")
mdweave_to_pdf(example1, "example1.pdf")
```

Although it is possible to pass additional arguments to `pandoc` through the
`mdweave_to_...` functions, it is probably just as easy to call `pandoc`
directly. 


## Writing markdown
An example of a basic code block tagged as being R-code is shown below. 
The id/label if the block is `codeblock1`.  

`````
```{#codeblock1 .R}
a <- 1+1
a
```
`````

By default the code in the code block is run and both the code and the output of
the code are shown in the generated new code block. In the output file this
will result in

```{#codeblock1 .R}
a <- 1+1
a
```

Note that the `.R` needs to be the first argument starting with a `.` for the
codeblock. For example ` ```{#codeblock .foo .R foo=bar} ` won't be evaluated,
while ` ```{#codeblock foo=bar .R .foo} ` will.

By default the code in the code block is run and both the code and the output of
the code are shown in the generated new code block. Arguments can be used to
suppress either showing the code or the output:

`````
```{#codeblock1 .R echo=FALSE results=TRUE}
a <- 1+1
```
`````

Inline code is also supported. Again it should be tagged with `{.R}`:

```
The average value of `Sepal.Width` is `mean(iris$Sepal.Width)`{.R} and 
that of `Petal.Width` is `mean(iris$Petal.Width)`{.R}.
```

The final result of the inline code will always be included as text into the
resulting markdown document. In case of code blocks the code is passed on to a
function. Depending on the function used this can result in a code blocks with
the evaluated code (the default), tables, figures and you can also specify your
own functions.

### Figures
To generate a figure use the `output_figure` function. The function will run
the code, capture any output on the specified device and generate a markdown
image include.

`````
```{.R #myfigure fun=output_figure name="test" caption="My figure" 
  device="pdf" width=8 height=6}
plot(dta$Sepal.Width, dta$Petal.Width)
```
`````

`output_figure` accepts the options `caption` (default none), `device` (possible
values are `"png"` and `"pdf"`), `echo` (default `FALSE`; echo the commands to
the output) and `results` (default `FALSE`; show output of the command (other
than the figure) in the output).  Additional arguments such as `width` and
`height` are passed on to the device (in this case `pdf()`). When an id is given
for the code block (as `#myfigure` above) this is also added to the figure.

The figures are saved in the folder `figures` in the current folder. This can be
controlled by the `dir` argument. 

### Tables
To generate a table, we tell it to pass the code in the code block to the
function `output_table` from the `simplermarkdown` package. This function will
take the final result and generate a markdown table from that. Any additional
arguments of the code block are passed on to the `output_table` function.

`````
```{.R #mytable fun=output_table caption="Sample iris"}
dta$foo <- dta$Sepal.Width/dta$Sepal.Length
dta[1:20, ]
```
`````

`output_table` only accepts the `caption` argument. Unfortunately, pandoc does
not yet support adding id's to tables at the moment without the
[pandoc crossref filter](https://github.com/lierdakil/pandoc-crossref).
Therefore, simplermarkdown also doesn't add the id to the resulting table as
this would interfere with regular pandoc use. Should you want to add an id to
the table in order to use it with the crossref filter: you can either do

````
: Sample iris {#mytable}

```{.R fun=output_table}
iris[1:5, ]
```
````
or 

````
```{.R fun=output_table caption="Sample iris {#mytable}"}
iris[1:5, ]
```
````



### Other output
By using the `output_raw` filter, any other output can be generated. This
function will run the code, capture any output and put that directly into the
resulting markdown document. For example, let's print a list with all of the
iris species:

`````
```{.R fun=output_raw}
writeLines(paste("-", levels(iris$Species)))
```
`````

Or you can write your own filter function. This function will get the code in
the code block as character vector as it's first argument, the language of the
code block, and the id of the code block and any other arguments given. 

For example:

```
print_in_bold <- function(code, language = "R", id = "", ...) {
  cat("\n**", code, "**\n", sep = "")
}
```

This function could for example be defined in a block in the markdown file.
Afterwards, this function can be used in your markdown document as:

`````
```{.R fun=print_in_bold}
Hello World!
```
`````


Using as a vignette engine
-----------------------------------------------------------------------

To use simplermarkdown as an engine for your R-package vignettes you will need to do the following:

#### Specify simplermarkdown as your vignette builder in your `DESCRIPTION` file:

```
VignetteBuilder: simplermarkdown
```

#### Add simplermarkdown as a dependency of your package. 
If your package doesn't use simplermarkdown otherwise you can add it to your
`Suggests` field in the `DESCRIPTION` file:

```
Suggests: 
    simplermarkdown
```

#### Use the extension `.md` for your vignette.
Create the vignette in the `vignettes` directory in your package source.

#### Specify the vignette engine in your vignette. 

You have to add the following line to your vignette:

```
%\VignetteEngine{simplermarkdown::mdweave_to_html}
```

Instead of `mdweave_to_html` you can also use `mdweave_to_pdf` to generate a
vignette in PDF format. It is easiest to do this in a comment section in your
markdown file. For example, start your markdown file with:

```
<!--
%\VignetteEngine{simplermarkdown::mdweave_to_html}
%\VignetteIndexEntry{The title of the vignette}
-->

---
title: [The title of the vignette]
---

[And the contents of your vignette]
```

### Custom styling
By default the default templates and styling of the pandoc installation on your
machine will be used.  However, you can also specify custom styling in the
header of your markdown file. See the
[documentation of Pandoc](https://pandoc.org/MANUAL.html) for more information.
For example, if you generate HTML output and you want to use a custom
CSS-stylesheet, you can place the stylesheet in the `vignettes` directory and
refer to the stylesheet in the header:

```
<!--
%\VignetteEngine{simplermarkdown::mdweave_to_html}
%\VignetteIndexEntry{The title of the vignette}
-->

---
title: [The title of the vignette]
css: custom_styling.css
---
```

## A note about paths and working directories
`simplermarkdown` tries to assume as little as possible about possible
workflows. However, this also means that you, the user, are responsible for some
things where other packages might make assumptions.  One of the places where
this is the case is for paths and working directories. And this is especially
relevant when including figures and when generating figures using R.

As an example take the following project directory:
`````
report/
   report.md
   figures/
     figure1.png
report/output/
`````

The report contains the following code:
`````
![Figure caption](figures/figure1.png)

```{.R fun=output_figure name="figure2" caption="Caption", device="png"}
plot(1:10)
```
`````

Assume the current working directory is the root of the project directory 
and that we run `mdweave` as:

`````
mdweave("report/report.md", "report/output/report.md")
`````
Figures are by default created in the directory `figures` in the 
target directory. Therefore the directory structure after running
`mdweave` is:

`````
report/
   report.md
   figures/
     figure1.png
   ouput/
     report.md
     figures/
       figure2.png
`````
And the resulting markdown file will contain the following markdown:
`````
![Figure caption](figures/figure1.png)

![Caption](report/output/figures/figure2.png)
`````

As you can see, we now have two locations with figures. When running 
`pandoc` from the root directory of the project to create the 
final output:
`````
pandoc report/output/report.md -s -o report/output/report.html
`````
`pandoc` will not be able to find the first figure. It will find the 
second figure.  When you would run the final `pandoc` command from 
the `report/output` directory. `pandoc` will not be able to find 
any of the figures.


There are several possible solutions for the example above:

- When working on linux or mac, you could create a symbolic link from
  `report/output/figures` to `report/figures`. 
- Copy `report/figures` to `report/output/figures`. 
- Path of least resistance: run `mdweave` and `pandoc` from the
  `report` directory and also put the output in the same directory.
- And probably others. 

Note that the same issues occur when referencing stylesheets etc. in the 
meta block of the markdown file.

---
title: An analysis of iris
---

```{.R results=FALSE echo=FALSE}
options(md_formatter = format_copypaste)
```

Introduction
-------------------------

From the help page of the iris data set:

> This famous (Fisher's or Anderson's) iris data set gives the
> measurements in centimeters of the variables sepal length and
> width and petal length and width, respectively, for 50 flowers
> from each of 3 species of iris.  The species are _Iris setosa_,
> _versicolor_, and _virginica_.


Descriptives
--------------------------

The table below shows for each of the iris species the mean value of the colums in the data set. 

```{.R #table fun=output_table caption="Mean values for each of the properties for each of the iris species."}
aggregate(iris[1:4], iris["Species"], mean)
```


```{.R #figure fun=output_figure 
  caption="Relation between sepal length and width for the different iris species." 
  name="iris" height=6 width=8 units="in" res=150 echo=TRUE}
pal <- hcl.colors(3, "Dark2")
plot(iris$Sepal.Width, iris$Sepal.Length, pch = 20, 
  col = pal[iris$Species], xlab = "Sepal Width", 
  ylab = "Sepal Length", bty = 'n', las = 1)
legend("topright", legend = levels(iris$Species), 
  fill = pal, bty = 'n', border = NA)
```



Species prediction
---------------------------------

```{.R}
library(MASS)
m <- lda(Species ~ Sepal.Width + Sepal.Length, data = iris)
p <- predict(m)
predicted_species <- p$class
table(predicted_species, iris$Species)
```

This model predicts in `round(mean(predicted_species==iris$Species)*100)`{.R}% of the
cases the correct species. However, this is mainly for *setosa* for the other species the
model predicts the correct species only for
`sel<-iris$Species!="setosa";round(100*mean(predicted_species[sel] == iris$Species[sel]))`{.R}% of
the records.



```{.R results=FALSE echo=FALSE}
options(md_formatter = NULL)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/md_table.R
\name{md_table}
\alias{md_table}
\title{Generate a markdown table from a data.frame}
\usage{
md_table(tab, caption, as_character = FALSE, ...)
}
\arguments{
\item{tab}{a data frame}

\item{caption}{text of the caption. When omitted no caption is added to the
table.}

\item{as_character}{return the table as a character vector. If \code{FALSE}
the table will be written to the standard output.}

\item{...}{unused.}
}
\value{
Then \code{as_character = FALSE} a character vector with the markdown
containing the table is returned.  Otherwise,
nothing is returned; the markdown is then written to the console.
}
\description{
Generate a markdown table from a data.frame
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file_subs_ext.R
\name{file_subs_ext}
\alias{file_subs_ext}
\title{Replace file extension by another file extension}
\usage{
file_subs_ext(fn, new_ext, check = TRUE)
}
\arguments{
\item{fn}{character vector with file names}

\item{new_ext}{character vector of length one with the new extension (when
it does not start with a period a period is added).}

\item{check}{check if the new file name is not equal to the original
filename. If so, generate an error.}
}
\value{
Returns a character vector of the same length of \code{fn} with the
extension of the file names in \code{fn} replaced by \code{new_ext}.
}
\description{
Replace file extension by another file extension
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdweave_to.R
\name{mdweave_to_pdf}
\alias{mdweave_to_pdf}
\alias{mdweave_to_tex}
\alias{mdweave_to_html}
\title{Run the code in a markdown file and generate a new document}
\usage{
mdweave_to_pdf(
  fn,
  ofn = file_subs_ext(basename(fn), ".pdf", FALSE),
  extra_arguments2 = "--self-contained",
  run_in_temp = TRUE,
  cmd2 = "pandoc \%3$s -s \\"\%1$s\\" -t latex -o \\"\%2$s\\"",
  ...
)

mdweave_to_tex(
  fn,
  ofn = file_subs_ext(basename(fn), ".tex", FALSE),
  extra_arguments2 = "--self-contained",
  run_in_temp = TRUE,
  cmd2 = "pandoc \%3$s -s \\"\%1$s\\" -t latex -o \\"\%2$s\\"",
  ...
)

mdweave_to_html(
  fn,
  ofn = file_subs_ext(basename(fn), ".html", FALSE),
  extra_arguments2 = "--self-contained",
  run_in_temp = TRUE,
  cmd2 = "pandoc \%3$s -s \\"\%1$s\\" -t html -o \\"\%2$s\\"",
  ...
)
}
\arguments{
\item{fn}{filename of the markdown file (should use pandoc markdown).}

\item{ofn}{name of the resulting file.}

\item{extra_arguments2}{extra arguments passed on to pandoc. Should be a length 1
character vector.}

\item{run_in_temp}{When TRUE the intermediary markdown file and generated figures (when
not using custom paths) are created in a temporary directory. Otherwise these will be
generated in the same directory as the output file.}

\item{cmd2}{command used to run pandoc. See details.}

\item{...}{additional arguments are passed on to \code{\link{mdweave}}.}
}
\value{
Returns the name of the resulting outout file.
}
\description{
Run the code in a markdown file and generate a new document
}
\details{
These functions first call \code{\link{mdweave}} to run the code in the original
file and convert the original markdown file to a new markdown file. This second
markdown file is then converted to the desired output format using a second run
of \code{pandoc}.

In case of converting to pdf the file is required to have the extension
\code{.pdf}. In case of converting to LaTeX, the file cannot have the extension
\code{.pdf}. That is because in both cases the file is first converted to
LaTeX. In case of a file with the extension \code{.pdf} the file is than
further converted to PDF.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_extensions.R
\name{get_extensions}
\alias{get_extensions}
\title{Get the extensions that are needed when converting from json to markdown by
pandoc.}
\usage{
get_extensions()
}
\value{
Returns a character vector of the form \code{"+extension1-extension2"} with
the extensions that are to be enabled (\code{+}) or disables (\code{-}).
The function calls \code{pandoc} to check which extensions are available.
}
\description{
Get the extensions that are needed when converting from json to markdown by
pandoc.
}
\details{
Checks which extensions are available in pandoc and returns a character
string disabling extensions that interfere with the conversion of the
processed document (in json format) back to markdown. At the moment only one
extension is disabled.

\describe{
\item{\code{raw_attribute}}{ This extension is disabled. Some of the output
functions write raw markdown blocks. This extension only recognises html
and tex raw blocks. Raw markdown blocks are ignored when converting to
other formats. }
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/run_and_capture.R
\name{run_and_capture}
\alias{run_and_capture}
\title{Run code and capture the output}
\usage{
run_and_capture(
  code,
  echo = TRUE,
  results = TRUE,
  output = results,
  capture_warnings = FALSE,
  capture_messages = results,
  muffle_warnings = FALSE,
  muffle_messages = TRUE
)
}
\arguments{
\item{code}{character vector or expression with the code to run}

\item{echo}{the code in \code{code} is repeated in the output.}

\item{results}{include the results of running the code in the output.}

\item{output}{include output that is explicitly written to the output, for
example using \code{print} statements.}

\item{capture_warnings}{include warnings in the output.}

\item{capture_messages}{include messages in the output.}

\item{muffle_warnings}{do not show warnings in the console.}

\item{muffle_messages}{do not show messages in the console.}
}
\value{
Returns a list. Each item of the list contains a list with elements
\code{input} and \code{output}. \code{input} contains the command/code and
\code{output} the corresponding output. These are empty vectors when there is
no output or when input and output are suppressed using one of the
\code{echo}/\code{results}/\code{output} statements.
}
\description{
Run code and capture the output
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/output_fun.R
\name{output_table}
\alias{output_table}
\alias{output_figure}
\alias{output_eval}
\alias{output_raw}
\alias{output_str}
\alias{output_shell}
\title{Output filters for code blocks in markdown}
\usage{
output_table(code, language = "R", id = "", ...)

output_figure(code, language = "R", id = "", ...)

output_eval(
  code,
  language = "R",
  id = "",
  echo = TRUE,
  results = TRUE,
  drop_empty = TRUE,
  eval = TRUE,
  formatter = getOption("md_formatter", default = format_traditional),
  capture_warnings = FALSE,
  capture_messages = results,
  muffle_warnings = FALSE,
  muffle_messages = TRUE,
  ...
)

output_raw(code, language = "R", id = "", ...)

output_str(code, language = "R", id = "", ...)

output_shell(
  code,
  language,
  id = "",
  cmd = language,
  echo = TRUE,
  results = TRUE,
  comment_char = "# ",
  ...
)
}
\arguments{
\item{code}{character vector containing the code of the code block}

\item{language}{the language in which the code is written (as specified in
the markdown file).}

\item{id}{the identifier of the code block.}

\item{...}{additional arguments specified as arguments in the code block are
passed on to the filter function. Often these are ignored, or they are
passed on to other functions (see 'Details').}

\item{echo}{the code in \code{code} is repeated in the output.}

\item{results}{include the results of running the code in the output. The
output of code that explicitly writes to standard output is always
included.}

\item{drop_empty}{do not include any output if the resulting code block
would be empty.}

\item{eval}{if FALSE do not run the code and just include the code in the
output.}

\item{formatter}{function that will format the R-code and resulting output
(if requested). See \code{\link{format_traditional}} for possible options.}

\item{capture_warnings}{include warnings in the output.}

\item{capture_messages}{include messages in the output.}

\item{muffle_warnings}{do not show warnings in the console.}

\item{muffle_messages}{do not show messages in the console.}

\item{cmd}{the command to use in the \code{\link{system2}} call. Only needed
when different than the language.}

\item{comment_char}{string prepended to output of commands run by
\code{output_shell}.}
}
\value{
The functions either return a character vector with markdown,
or return a list with the correct structure to include in the pandoc parse
tree.
}
\description{
Output filters for code blocks in markdown
}
\details{
The filter functions \code{output_table} and \code{output_figure} call
\code{\link{md_table}} and \code{\link{md_figure}} respectively; additional
arguments are passed on to those functions. Other filter functions ignore the
additional arguments.

It is also possible to write custom output filter. An output filter should have
\code{code}, \code{language} and \code{id} as its first three arguments. It
should either return a character vector containing the markdown that should be
included in the resulting markdown file or an object that can be directly
included in the pandoc parse tree. If the function does not return a character
vector it is assumed the latter is returned. \code{simplermarkdown} defines a
small number of valid object constructors: \code{\link{raw_block}} and
\code{\link{markdown_block}}.

The custom function should be available when running the markdown document
through pandoc. The easiest way is to \code{\link{source}} or define the
function in the markdown document before using it.

The filter function \code{output_shell} can be used to process chunks of code
from other languages than R. These chunks of code are written to a temporary
file which is then ran using the \code{cmd} using a call to
\code{\link{system2}}. The output of that is captured and when \code{results
= TRUE} included in the output. The output lines are prepended by
\code{comment_char}. When \code{echo = TRUE} the code is also included before
the output.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdtangle.R
\name{mdtangle}
\alias{mdtangle}
\title{Extract code from the code blocks in a markdown file}
\usage{
mdtangle(
  fn,
  ofn = file_subs_ext(basename(fn), ".R"),
  extra_arguments = "",
  cmd = "pandoc \%3$s -s \\"\%1$s\\" -t json -o \\"\%2$s\\"",
  ...
)
}
\arguments{
\item{fn}{filename of the markdown file (should use pandoc markdown).}

\item{ofn}{name of the resulting R-script}

\item{extra_arguments}{extra arguments passed on to pandoc. Should be a length 1
character vector.}

\item{cmd}{command used to run pandoc. See details.}

\item{...}{ignored}
}
\value{
Returns the filename of the generated file.
}
\description{
Extract code from the code blocks in a markdown file
}
\details{
\code{mdtangle} calls pandoc. Pandoc will parse the markdown document and
write the parsed file to temporary file. This file is read by
\code{mdtangle} and the code is extracted from it and written to \code{ofn}.

Using the \code{cmd} argument the exact command used to run pandoc can be
modified. It is passed on to \code{\link{sprintf}} and uses positional
arguments: (1) name of the input file, (2) location of the temporary file
to which the parsed document is written, (3) the value of
\code{extra_arguments}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mdweave.R
\name{mdweave}
\alias{mdweave}
\title{Run the code in a markdown file and generate a new markdown file}
\usage{
mdweave(
  fn,
  ofn = file_subs_ext(basename(fn), ".md", FALSE),
  cmd1 = "pandoc -s \\"\%1$s\\" -t json -o \\"\%2$s\\"",
  cmd2 = "pandoc -s \\"\%1$s\\" -t markdown\%3$s -o \\"\%2$s\\"",
  ...
)
}
\arguments{
\item{fn}{filename of the markdown file (should use pandoc markdown).}

\item{ofn}{name of the resulting markdown file.}

\item{cmd1}{command used to run pandoc. See details.}

\item{cmd2}{command used to run pandoc. See details.}

\item{...}{ignored}
}
\value{
Returns the file name of the file generated (\code{ofn}). Called mainly for the
side effect of parsing and generating a markdown file (and possibly secondary
files such as figures).
}
\description{
Run the code in a markdown file and generate a new markdown file
}
\details{
\code{mdweave} calls pandoc twice. In the first call the markdown file is
parsed by pandoc and the parse tree is written to a temporary file. This
parse tree is the read by mdweave and any R-code in the tree is executed
resulting in a modified parse tree. This file is then stored to a new
temporary file. Pandoc is the called a second time to convert the new
parse tree to a markdown file.

The arguments \code{cmd1} and \code{cmd2} contain the calls used to run
pandoc. The arguments can be used to, for example pas additional arguments
to pandoc. They use positional arguments. In \code{cmd1}, the first argument
(\code{\%1$s}) is the input file name and the second (\code{\%2$s}) the
temporary file containing the parsed tree. In \code{cmd2}, the first argument
is the temporary file with the modified parse tree and the second argument
the output file.

\code{cmd2} also has a third argument (\code{\%3$s}) that contains a list of
extensions that are enabled or disabled. This is because some extensions
interfere with the conversion of the parsed tree to markdown. See
\code{\link{get_extensions}} to see which extensions are disabled.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/markdown_block.R
\name{markdown_block}
\alias{markdown_block}
\title{Return a code block object that can be included in the pandoc parse tree}
\usage{
markdown_block(content, language, id = "", ...)
}
\arguments{
\item{content}{a character vector containing the code}

\item{language}{language of the code in the code block}

\item{id}{optional id of the code block}

\item{...}{additional arguments should be named. These are added to the
markdown block as additional arguments.}
}
\value{
Returns a \code{list} with the correct structure for a code block in
the pandoc parse tree.
}
\description{
Return a code block object that can be included in the pandoc parse tree
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/md_figure.R
\name{md_figure}
\alias{md_figure}
\title{Generate a figure and generate the markdown to include the figure}
\usage{
md_figure(
  expr,
  name,
  caption = "",
  id = "",
  dir = file.path(Sys.getenv("MDOUTDIR", "."), "figures"),
  device = c("png", "pdf"),
  ...,
  as_character = FALSE,
  echo = FALSE,
  results = FALSE,
  formatter = getOption("md_formatter", default = format_traditional),
  capture_warnings = FALSE,
  capture_messages = results,
  muffle_warnings = FALSE,
  muffle_messages = TRUE
)
}
\arguments{
\item{expr}{the expressions to evaluate. Will generally contain plotting
commands. The expressions are evaluated in the global environment.}

\item{name}{the name of the figure.}

\item{caption}{text of the caption. When omitted no caption is added to the
figure.}

\item{id}{id of the figure. When omitted or equal to NULL or an empty
character, no id is added to the figure.}

\item{dir}{name of the directory in which to store the file.}

\item{device}{the graphics device to use for creating the image.}

\item{...}{passed on to the graphics device.}

\item{as_character}{return the figure as a character vector. If \code{FALSE}
the figure will be written to the standard output.}

\item{echo}{the code in \code{code} is repeated in the output.}

\item{results}{include the results of running the code in the output. The
output of code that explicitly writes to standard output is always
included.}

\item{formatter}{function that will format the R-code and resulting output
(if requested). See \code{\link{format_traditional}} for possible options.}

\item{capture_warnings}{include warnings in the output.}

\item{capture_messages}{include messages in the output.}

\item{muffle_warnings}{do not show warnings in the console.}

\item{muffle_messages}{do not show messages in the console.}
}
\value{
When \code{as_character = FALSE} a character vector with the markdown needed
to include the generated figure in a markdown file is returned. Otherwise,
nothing is returned; the markdown is written to the console.
}
\description{
Will evaluate the expressions in \code{expr} and capture the output on the
given plotting device in the given file. It will then generate the markdown
needed to include that figure in a markdown document.
}
\details{
The image is stored in the file \code{dir/name.device}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/raw_block.R
\name{raw_block}
\alias{raw_block}
\title{Return a raw chunk of text that can be included in the pandoc parse tree}
\usage{
raw_block(content, language = "markdown")
}
\arguments{
\item{content}{a character vector containing the content to include
in the final document.}

\item{language}{language of the content}
}
\value{
Returns a \code{list} with the correct structure for a \code{RawBlock} in
the pandoc parse tree.
}
\description{
Return a raw chunk of text that can be included in the pandoc parse tree
}
\details{
A raw block is included as is into the final markdown document. This can
be used for example to include raw chunks of markdown.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_eval.R
\name{format_traditional}
\alias{format_traditional}
\alias{format_copypaste}
\title{Format the result of running a block of code}
\usage{
format_traditional(x)

format_copypaste(x)
}
\arguments{
\item{x}{the result of running the code. See 'details' for the format.}
}
\value{
A character vector of length 1 with the formatted code.
}
\description{
Format the result of running a block of code
}
\details{
The input vector \code{x} should be a list. Each element of this list should
be a list with two elements: \code{input} and \code{output}. \code{input}
should contain the evaluated expression; this can be more than one line.
\code{output} should contain the output of the evaluation. When there is no
output this is character vector of length 0.
}
